x,  y  = 0.1, 0.5
x1, y1 = 0.4, 0.9
x2, y2 = 0.6, 0.1
x3, y3 = 0.9, 0.5

snippet_normalize (cr, width, height)

cr.move_to (x, y)
cr.curve_to (x1, y1, x2, y2, x3, y3)

cr.stroke ()

cr.set_source_rgba (1,0.2,0.2,0.6)
cr.set_line_width (0.03)
cr.move_to (x,y);   cr.line_to (x1,y1)
cr.move_to (x2,y2); cr.line_to (x3,y3)
cr.stroke ()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import wx
import wx.adv

phoenix = ("""\
A phoenix is a mythical bird with a colorful plumage and a tail of gold and scarlet
(or purple and blue, according to some sources). It has a 500 to 1,000 year life-cycle,
near the end of which it builds itself a nest of myrrh twigs that then ignites; both nest
and bird burn fiercely and are reduced to ashes, from which a new, young phoenix or phoenix
egg arises, reborn anew to live again. The new phoenix is destined to live as long as its old
self. """)

class SampleBanners(wx.Panel):
    def __init__(self, parent):
        # ... create the frame itself ...
        wx.Panel.__init__(self, parent)

        pnxBmp = wx.Bitmap('bitmaps/phoenix_top.png')
        bmpsz = pnxBmp.GetSize()

        # Create and initialize the banner.
        whitePanel = wx.Panel(self, -1, size=(-1, bmpsz[1]))
        whitePanel.SetBackgroundColour(wx.WHITE)

        # Create and initialize the 1st banner and define a bitmap.
        banner1 = wx.adv.BannerWindow(whitePanel, dir=wx.BOTTOM)
        banner1.SetBitmap(pnxBmp)

        whiteSizer = wx.BoxSizer(wx.HORIZONTAL)
        whiteSizer.Add(banner1, 1)
        whitePanel.SetSizer(whiteSizer)

        # Create and initialize the 2nd banner and define the gradient text.
        banner2 = wx.adv.BannerWindow(self, dir=wx.TOP)
        banner2.SetGradient(start='#FF8000', end='#FFFFFF')
        banner2.SetText("Phoenix", phoenix)

        # Layout
        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(whitePanel, 0, wx.EXPAND | wx.TOP | wx.LEFT | wx.RIGHT, 5)
        vsizer.Add(banner2, 1, wx.EXPAND | wx.BOTTOM | wx.LEFT | wx.RIGHT, 5)

        self.SetSizer(vsizer)



class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        banners = SampleBanners(self)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(banners, 0, wx.EXPAND)

        text = wx.StaticText(self, -1, overview)
        sizer.Add(text, 0, wx.EXPAND|wx.ALL, 15)

        self.SetSizer(sizer)




def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win


#---------------------------------------------------------------------------


overview = """\
This sample displays two banner windows, one with an image, and one with text and
a colour gradient background.

Banner windows can be used to present some overview of the current window contents
to the user in an aesthetically pleasant way. They are typically positioned along
the left or top edge of the window (although this class also supports right-aligned
and bottom-aligned banners) and show either a bitmap with a logo or a few lines of
text on a gradient-filled background.
"""


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#!/usr/bin/env python

import  wx
import  wx.xrc  as  xrc

#----------------------------------------------------------------------

resourceText = br'''<?xml version="1.0"?>
<resource>

<!-- Notice that the class is NOT a standard wx class -->

<object class="MyCustomPanel" name="MyPanel">
    <size>200,100</size>
    <object class="wxStaticText" name="label1">
        <label>This panel is a custom class derived from wx.Panel,\nand is loaded by a custom XmlResourceHandler.</label>
        <pos>10,10</pos>
    </object>
</object>
</resource>
'''

#----------------------------------------------------------------------

class MyCustomPanel(wx.Panel):
    def __init__(self, parent, id, pos, size, style, name):
        wx.Panel.__init__(self, parent, id, pos, size, style, name)

        # This is the little bit of customization that we do for this
        # silly example.
        self.Bind(wx.EVT_SIZE, self.OnSize)
        t = wx.StaticText(self, -1, "MyCustomPanel")
        f = t.GetFont()
        f.SetWeight(wx.FONTWEIGHT_BOLD)
        f.SetPointSize(f.GetPointSize()+2)
        t.SetFont(f)
        self.t = t

    def OnSize(self, evt):
        sz = self.GetSize()
        w, h = self.t.GetTextExtent(self.t.GetLabel())
        self.t.SetPosition(((sz.width-w)/2, (sz.height-h)/2))


# To do it the more complex way, (see below) we need to write the
# class a little differently...  This could obviously be done with a
# single class, but I wanted to make separate ones to make clear what
# the different requirements are.
class PreMyCustomPanel(wx.Panel):
    def __init__(self):
        wx.Panel.__init__(self)

    def Create(self, parent, id, pos, size, style, name):
        wx.Panel.Create(self, parent, id, pos, size, style, name)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        t = wx.StaticText(self, -1, "MyCustomPanel")
        f = t.GetFont()
        f.SetWeight(wx.FONTWEIGHT_BOLD)
        f.SetPointSize(f.GetPointSize()+2)
        t.SetFont(f)
        self.t = t

    def OnSize(self, evt):
        sz = self.GetSize()
        w, h = self.t.GetTextExtent(self.t.GetLabel())
        self.t.SetPosition(((sz.width-w)/2, (sz.height-h)/2))

#----------------------------------------------------------------------

class MyCustomPanelXmlHandler(xrc.XmlResourceHandler):
    def __init__(self):
        xrc.XmlResourceHandler.__init__(self)
        # Specify the styles recognized by objects of this type
        self.AddStyle("wxTAB_TRAVERSAL", wx.TAB_TRAVERSAL)
        self.AddStyle("wxWS_EX_VALIDATE_RECURSIVELY", wx.WS_EX_VALIDATE_RECURSIVELY)
        self.AddStyle("wxCLIP_CHILDREN", wx.CLIP_CHILDREN)
        self.AddWindowStyles()

    # This method and the next one are required for XmlResourceHandlers
    def CanHandle(self, node):
        return self.IsOfClass(node, "MyCustomPanel")

    def DoCreateResource(self):
        # NOTE: wxWindows can be created in either a single-phase or
        # in a two-phase way.  Single phase is what you normally do,
        # and two-phase creates the instance first, and then later
        # creates the actual window when the Create method is called.
        #
        # wxXmlResource supports either method, a pre-made instance can
        # be created and populated by xrc using the appropriate
        # LoadOn* method (such as LoadOnPanel) or xrc can create the
        # instance too, using the Load* method.  However this makes
        # the handlers a bit more complex.  If you can be sure that a
        # particular class will never be loaded using a pre-existing
        # instance, then you can make the handle much simpler.  I'll
        # show both methods below.

        if 0:
            # The simple method assumes that there is no existing
            # instance.  Be sure of that with an assert.
            assert self.GetInstance() is None

            # Now create the object
            panel = MyCustomPanel(self.GetParentAsWindow(),
                                  self.GetID(),
                                  self.GetPosition(),
                                  self.GetSize(),
                                  self.GetStyle("style", wx.TAB_TRAVERSAL),
                                  self.GetName()
                                  )
        else:
            # When using the more complex (but more flexible) method
            # the instance may already have been created, check for it
            panel = self.GetInstance()
            if panel is None:
                # if not, then create the instance (but not the window)
                panel = PreMyCustomPanel()

            # Now call the panel's Create method to actually create the window
            panel.Create(self.GetParentAsWindow(),
                         self.GetID(),
                         self.GetPosition(),
                         self.GetSize(),
                         self.GetStyle("style", wx.TAB_TRAVERSAL),
                         self.GetName()
                         )

        # These two things should be done in either case:
        # Set standard window attributes
        self.SetupWindow(panel)
        # Create any child windows of this node
        self.CreateChildren(panel)

        return panel


#----------------------------------------------------------------------


class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        # make the components
        label = wx.StaticText(self, -1, "The lower panel was built from this XML:")
        label.SetFont(wx.Font(12, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))

        text = wx.TextCtrl(self, -1, resourceText,
                          style=wx.TE_READONLY|wx.TE_MULTILINE)
        text.SetInsertionPoint(0)

        line = wx.StaticLine(self, -1)

        # Load the resource
        res = xrc.XmlResource()
        res.InsertHandler(MyCustomPanelXmlHandler())
        res.LoadFromBuffer(resourceText)

        # Now create a panel from the resource data
        panel = res.LoadObject(self, "MyPanel", "MyCustomPanel")

        # and do the layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(label, 0, wx.EXPAND|wx.TOP|wx.LEFT, 5)
        sizer.Add(text, 1, wx.EXPAND|wx.ALL, 5)
        sizer.Add(line, 0, wx.EXPAND)
        sizer.Add(panel, 1, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(sizer)


#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------



overview = """<html><body>
<h2><center>wx.XmlResourceHandler</center></h2>

Deriving a class from wx.XmlResourceHandler allows you to specify your
own classes in XRC resources, and your handler class will then be used
to create instances of that class when the resource is loaded.

</body></html>
"""



if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#---------------------------------------------------------------------------
# Name:        etg/palette.py
# Author:      Scott Talbert
#
# Created:     31-Jul-2016
# Copyright:   (c) 2016-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools
from textwrap import dedent

PACKAGE   = "wx"
MODULE    = "_core"
NAME      = "palette"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxPalette' ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.
    c = module.find('wxPalette')
    tools.removeVirtuals(c)


    # Add a helper function for use with the Create method and the Ctor
    # accepting RGB data.
    c.addCppCode("""\
        // a helper function to be used in the Create method and one of the Ctors
        bool _paletteCreateHelper(wxPalette* self,
                                  PyObject* red, PyObject* green, PyObject* blue) {
            bool rval = false;
            wxPyThreadBlocker blocker;
            char* errMsg = "Expected a sequence of integer objects";

            if (!PySequence_Check(red) || !PySequence_Check(green) || !PySequence_Check(blue)) {
                PyErr_SetString(PyExc_TypeError, errMsg);
                return rval;
            }

            Py_ssize_t count = PySequence_Size(red);
            if (PySequence_Size(green) != count || PySequence_Size(blue) != count) {
                PyErr_SetString(PyExc_ValueError, "Sequence lengths must be equal");
                return rval;
            }

            unsigned char* redArray = new unsigned char[count];
            unsigned char* greenArray = new unsigned char[count];
            unsigned char* blueArray = new unsigned char[count];

            for (Py_ssize_t i = 0; i < count; i++) {
                PyObject* redItem = PySequence_ITEM(red, i);
                PyObject* greenItem = PySequence_ITEM(green, i);
                PyObject* blueItem = PySequence_ITEM(blue, i);
                if (!wxPyInt_Check(redItem) || !wxPyInt_Check(greenItem) || !wxPyInt_Check(blueItem)) {
                    PyErr_SetString(PyExc_TypeError, errMsg);
                    goto pch_exit;
                }

                long redLong = wxPyInt_AsLong(redItem);
                long greenLong = wxPyInt_AsLong(greenItem);
                long blueLong = wxPyInt_AsLong(blueItem);
                Py_DECREF(redItem);
                Py_DECREF(greenItem);
                Py_DECREF(blueItem);
                if (redLong < 0 || redLong > 255 || greenLong < 0 || greenLong > 255 || blueLong < 0 || blueLong > 255) {
                    PyErr_SetString(PyExc_ValueError, "Sequence values must be in the 0..255 range");
                    goto pch_exit;
                }
                redArray[i] = redLong;
                greenArray[i] = greenLong;
                blueArray[i] = blueLong;
            }

            rval = self->Create(count, redArray, greenArray, blueArray);

        pch_exit:
            delete[] redArray;
            delete[] greenArray;
            delete[] blueArray;
            return rval;
        }
        """)


    #-----------------------------------------------------------------

    # Replace the constructor accepting the arrays of RGB values with one that
    # understands any Python sequence.
    c.find('wxPalette').findOverload('red').ignore()
    c.addCppCtor('(PyObject* red, PyObject* green, PyObject* blue)',
        doc=dedent("""\
        Creates a palette from a set of sequences of integers,
        one for each red, green and blue color components.

        :param red: A sequence of integer values in the range 0..255 inclusive.
        :param green: A sequence of integer values in the range 0..255 inclusive.
        :param blue: A sequence of integer values in the range 0..255 inclusive.

        .. note:: All sequences must be the same length.
        """),
        body="""\
            wxPalette* pal = new wxPalette;
            _paletteCreateHelper(pal, red, green, blue);
            if (PyErr_Occurred()) {
                delete pal;
                return NULL;
            }
            return pal;
            """)

    c.find('GetPixel.red').pyInt = True
    c.find('GetPixel.green').pyInt = True
    c.find('GetPixel.blue').pyInt = True

    c.find('GetRGB').ignore()
    c.addCppMethod('PyObject*', 'GetRGB', '(int pixel)',
        pyArgsString="(pixel) -> (red, green, blue)",
        doc="Returns RGB values for a given palette index.",
        body="""\
            unsigned char red;
            unsigned char green;
            unsigned char blue;
            wxPyThreadBlocker blocker;
            if (!self->GetRGB(pixel, &red, &green, &blue)) {
                PyErr_SetString(PyExc_IndexError, "Pixel out of range");
                return NULL;
            }
            PyObject* rv = PyTuple_New(3);
            PyTuple_SetItem(rv, 0, wxPyInt_FromLong(red));
            PyTuple_SetItem(rv, 1, wxPyInt_FromLong(green));
            PyTuple_SetItem(rv, 2, wxPyInt_FromLong(blue));
            return rv;
            """)

    # Replace the Create method with one that understands any kind of Python sequence
    c.find('Create').ignore()
    c.addCppMethod('PyObject*', 'Create', '(PyObject* red, PyObject* green, PyObject* blue)',
        pyArgsString="(red, green, blue) -> bool",
        doc=dedent("""\
        Creates a palette from 3 sequences of integers, one for each red, blue or green component.

        :param red: A sequence of integer values in the range 0..255 inclusive.
        :param green: A sequence of integer values in the range 0..255 inclusive.
        :param blue: A sequence of integer values in the range 0..255 inclusive.

        .. note:: All sequences must be the same length.
        """),
        body="""\
            bool rval = _paletteCreateHelper(self, red, green, blue);
            wxPyThreadBlocker blocker;
            if (PyErr_Occurred())
                return NULL;
            if (rval)
                Py_RETURN_TRUE;
            else
                Py_RETURN_FALSE;
            """)


    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()



#!/usr/bin/env python

import  wx

import ColorPanel
import images

colourList = [ "Aquamarine", "Grey", "Blue", "Blue Violet", "Brown", "Cadet Blue",
               "Coral", "Wheat", "Cyan", "Dark Grey", "Dark Green",
               "Steel Blue",
               ]

#----------------------------------------------------------------------------

def getNextImageID(count):
    imID = 0
    while True:
        yield imID
        imID += 1
        if imID == count:
            imID = 0


class TestTB(wx.Treebook):
    def __init__(self, parent, id, log):
        wx.Treebook.__init__(self, parent, id, style=
                             wx.BK_DEFAULT
                             #wx.BK_TOP
                             #wx.BK_BOTTOM
                             #wx.BK_LEFT
                             #wx.BK_RIGHT
                            )
        self.log = log

        # make an image list using the LBXX images
        il = wx.ImageList(32, 32)
        for x in range(12):
            obj = getattr(images, 'LB%02d' % (x+1))
            bmp = obj.GetBitmap()
            il.Add(bmp)
        self.AssignImageList(il)
        imageIdGenerator = getNextImageID(il.GetImageCount())

        # Now make a bunch of panels for the list book
        first = True
        for colour in colourList:
            win = self.makeColorPanel(colour)
            self.AddPage(win, colour, imageId=next(imageIdGenerator))
            if first:
                st = wx.StaticText(win.win, -1,
                          "You can put nearly any type of window here,\n"
                          "and the wx.TreeCtrl can be on either side of the\n"
                          "Treebook",
                          wx.Point(10, 10))
                first = False

            win = self.makeColorPanel(colour)
            st = wx.StaticText(win.win, -1, "this is a sub-page", (10,10))
            self.AddSubPage(win, 'a sub-page', imageId=next(imageIdGenerator))

        self.Bind(wx.EVT_TREEBOOK_PAGE_CHANGED, self.OnPageChanged)
        self.Bind(wx.EVT_TREEBOOK_PAGE_CHANGING, self.OnPageChanging)

        # This is a workaround for a sizing bug on Mac...
        wx.CallLater(100, self.AdjustSize)

    def AdjustSize(self):
        self.PostSizeEvent()

    def makeColorPanel(self, color):
        p = wx.Panel(self, -1)
        win = ColorPanel.ColoredPanel(p, color)
        p.win = win
        def OnCPSize(evt, win=win):
            win.SetPosition((0,0))
            win.SetSize(evt.GetSize())
        p.Bind(wx.EVT_SIZE, OnCPSize)
        return p

    def OnPageChanged(self, event):
        old = event.GetOldSelection()
        new = event.GetSelection()
        sel = self.GetSelection()
        self.log.write('OnPageChanged,  old:%d, new:%d, sel:%d\n' % (old, new, sel))
        event.Skip()

    def OnPageChanging(self, event):
        old = event.GetOldSelection()
        new = event.GetSelection()
        sel = self.GetSelection()
        self.log.write('OnPageChanging, old:%d, new:%d, sel:%d\n' % (old, new, sel))
        event.Skip()

#----------------------------------------------------------------------------

def runTest(frame, nb, log):
    testWin = TestTB(nb, -1, log)
    return testWin

#----------------------------------------------------------------------------


overview = """\
<html><body>
<h2>wx.Treebook</h2>
<p>
This class is a control similar to a notebook control, but with a
wx.TreeCtrl instead of a set of tabs.

"""


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#!/usr/bin/env python

import wx

import os
import sys

try:
    dirName = os.path.dirname(os.path.abspath(__file__))
except:
    dirName = os.path.dirname(os.path.abspath(sys.argv[0]))

sys.path.append(os.path.split(dirName)[0])

try:
    from agw import multidirdialog as MDD
except ImportError: # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.multidirdialog as MDD

import images


class MultiDirDialogDemo(wx.Panel):

    def __init__(self, parent, log):

        wx.Panel.__init__(self, parent)

        self.log = log

        self.mainPanel = wx.Panel(self)
        self.dialogSizer_staticbox = wx.StaticBox(self.mainPanel, -1, "Dialog Styles")

        self.dd_multiple = wx.CheckBox(self.mainPanel, -1, "MDD.DD_MULTIPLE")
        self.dd_must = wx.CheckBox(self.mainPanel, -1, "wx.DD_DIR_MUST_EXIST")
        self.dd_new = wx.CheckBox(self.mainPanel, -1, "wx.DD_NEW_DIR_BUTTON")

        self.showDialog = wx.Button(self.mainPanel, -1, "Show MultiDirDialog")

        self.SetProperties()
        self.DoLayout()

        self.Bind(wx.EVT_BUTTON, self.OnShowDialog, self.showDialog)


    def SetProperties(self):

        self.showDialog.SetDefault()


    def DoLayout(self):

        frameSizer = wx.BoxSizer(wx.VERTICAL)
        panelSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer = wx.BoxSizer(wx.HORIZONTAL)
        buttonSizer = wx.StaticBoxSizer(self.dialogSizer_staticbox, wx.VERTICAL)
        buttonSizer.Add(self.dd_multiple, 0, wx.LEFT|wx.RIGHT|wx.TOP, 5)
        buttonSizer.Add((0, 2), 0, 0, 0)
        buttonSizer.Add(self.dd_must, 0, wx.LEFT|wx.RIGHT, 5)
        buttonSizer.Add((0, 2), 0, 0, 0)
        buttonSizer.Add(self.dd_new, 0, wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)

        mainSizer.Add(buttonSizer, 0, wx.ALL|wx.EXPAND, 5)
        mainSizer.Add((10, 0), 0, 0, 0)
        mainSizer.Add(self.showDialog, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 5)
        mainSizer.Add((10, 0), 0, 0, 0)

        panelSizer.Add(mainSizer, 0, wx.EXPAND)
        self.mainPanel.SetSizer(panelSizer)

        frameSizer.Add(self.mainPanel, 1, wx.EXPAND)
        self.SetSizer(frameSizer)
        frameSizer.Layout()


    def OnShowDialog(self, event):

        dlgStyle = 0
        for child in self.mainPanel.GetChildren():
            if isinstance(child, wx.CheckBox):
                if child.GetValue():
                    dlgStyle |= eval(child.GetLabel())

        userPath = wx.StandardPaths.Get().GetUserDataDir()
        dlg = MDD.MultiDirDialog(self, title="Custom MultiDirDialog :-D", defaultPath=userPath,
                                 agwStyle=dlgStyle)

        dlg.SetIcon(images.Mondrian.GetIcon())
        if dlg.ShowModal() != wx.ID_OK:
            self.log.write("You Cancelled The Dialog!\n")
            dlg.Destroy()
            return

        paths = dlg.GetPaths()
        for indx, path in enumerate(paths):
            self.log.write("Path %d: %s\n"%(indx+1, path))

        dlg.Destroy()


#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = MultiDirDialogDemo(nb, log)
    return win

#----------------------------------------------------------------------


overview = MDD.__doc__


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#!/usr/bin/env python

import wx
from Main import opj


#----------------------------------------------------------------------

msg = "Some text will appear mixed in the image's shadow..."

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        self.Bind(wx.EVT_PAINT, self.OnPaint)


    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush("WHITE"))
        dc.Clear()

        dc.SetFont(wx.Font(16, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, True))
        dc.DrawText("Bitmap alpha blending (on all ports but gtk+ 1.2)",
                    25,25)

        bmp = wx.Bitmap(opj('bitmaps/toucan.png'))
        if "gtk1" in wx.PlatformInfo:
            # Try to make up for lack of alpha support in wxGTK (gtk+
            # 1.2) by converting the alpha blending into a
            # transparency mask.

            # first convert to a wx.Image
            img = bmp.ConvertToImage()

            # Then convert the alpha channel to a mask, specifying the
            # threshold below which alpha will be made fully
            # transparent
            img.ConvertAlphaToMask(220)

            # convert back to a wx.Bitmap
            bmp = img.ConvertToBitmap()


        dc.DrawBitmap(bmp, 25,100, True)

        dc.SetFont(self.GetFont())
        y = 75
        for line in range(10):
            y += dc.GetCharHeight() + 5
            dc.DrawText(msg, 200, y)
        dc.DrawBitmap(bmp, 250,100, True)



#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------



overview = """<html><body>
<h2><center>Images with Alpha</center></h2>

wxMSW and wxMac now support alpha channels of supported image
types, and will properly blend that channel when drawing a
bitmap.  It is not supported yet on wxGTK, (if you would like to
change that please submit a patch!)

<p>On wxGTK this demo turns the alpha channel into a 1-bit mask, so
yes, it looks like crap.  Please help us fix it!

</body></html>
"""



if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])





                text = wx.StaticText(self, -1, 'Hello world!')

                # Some more code...
                text.SetLabelMarkup("<b>&ampBed</b> &ampmp "
                                     "<span foreground='red'>breakfast</span> "
                                     "available <big>HERE</big>")




import wx
import wx.stc as stc

#----------------------------------------------------------------------

debug = 1


demoText = """\
This editor is provided by a class named wx.StyledTextCtrl.  As
the name suggests, you can define styles that can be applied to
sections of text.  This will typically be used for things like
syntax highlighting code editors, but I'm sure that there are other
applications as well.  A style is a combination of font, point size,
foreground and background colours.  The editor can handle
proportional fonts just as easily as monospaced fonts, and various
styles can use different sized fonts.

There are a few canned language lexers and colourizers included,
(see the next demo) or you can handle the colourization yourself.
If you do you can simply register an event handler and the editor
will let you know when the visible portion of the text needs
styling.

wx.StyledTextCtrl also supports setting markers in the margin...




...and indicators within the text.  You can use these for whatever
you want in your application.  Cut, Copy, Paste, Drag and Drop of
text works, as well as virtually unlimited Undo and Redo
capabilities, (right click to try it out.)
"""

if wx.Platform == '__WXMSW__':
    face1 = 'Arial'
    face2 = 'Times New Roman'
    face3 = 'Courier New'
    pb = 10
else:
    face1 = 'Helvetica'
    face2 = 'Times'
    face3 = 'Courier'
    pb = 12


#----------------------------------------------------------------------
# This shows how to catch the Modified event from the wx.StyledTextCtrl

class MySTC(stc.StyledTextCtrl):
    def __init__(self, parent, ID, log):
        stc.StyledTextCtrl.__init__(self, parent, ID)
        self.log = log

        self.Bind(stc.EVT_STC_DO_DROP, self.OnDoDrop)
        self.Bind(stc.EVT_STC_DRAG_OVER, self.OnDragOver)
        self.Bind(stc.EVT_STC_START_DRAG, self.OnStartDrag)
        self.Bind(stc.EVT_STC_MODIFIED, self.OnModified)

        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy)

    def OnDestroy(self, evt):
        # This is how the clipboard contents can be preserved after
        # the app has exited.
        if 'wxMac' not in wx.PlatformInfo:
            wx.TheClipboard.Flush()
        evt.Skip()


    def OnStartDrag(self, evt):
        self.log.write("OnStartDrag: %d, %s\n"
                       % (evt.GetDragAllowMove(), evt.GetDragText()))

        if debug and evt.GetPosition() < 250:
            evt.SetDragAllowMove(False)     # you can prevent moving of text (only copy)
            evt.SetDragText("DRAGGED TEXT") # you can change what is dragged
            #evt.SetDragText("")             # or prevent the drag with empty text


    def OnDragOver(self, evt):
        self.log.write(
            "OnDragOver: x,y=(%d, %d)  pos: %d  DragResult: %d\n"
            % (evt.GetX(), evt.GetY(), evt.GetPosition(), evt.GetDragResult())
            )

        if debug and evt.GetPosition() < 250:
            evt.SetDragResult(wx.DragNone)   # prevent dropping at the beginning of the buffer


    def OnDoDrop(self, evt):
        self.log.write("OnDoDrop: x,y=(%d, %d)  pos: %d  DragResult: %d\n"
                       "\ttext: %s\n"
                       % (evt.GetX(), evt.GetY(), evt.GetPosition(), evt.GetDragResult(),
                          evt.GetDragText()))

        if debug and evt.GetPosition() < 500:
            evt.SetDragText("DROPPED TEXT")  # Can change text if needed
            #evt.SetDragResult(wx.DragNone)  # Can also change the drag operation, but it
                                             # is probably better to do it in OnDragOver so
                                             # there is visual feedback

            #evt.SetPosition(25)             # Can also change position, but I'm not sure why
                                             # you would want to...




    def OnModified(self, evt):
        self.log.write("""OnModified
        Mod type:     %s
        At position:  %d
        Lines added:  %d
        Text Length:  %d
        Text:         %s\n""" % ( self.transModType(evt.GetModificationType()),
                                  evt.GetPosition(),
                                  evt.GetLinesAdded(),
                                  evt.GetLength(),
                                  repr(evt.GetText()) ))


    def transModType(self, modType):
        st = ""
        table = [(stc.STC_MOD_INSERTTEXT, "InsertText"),
                 (stc.STC_MOD_DELETETEXT, "DeleteText"),
                 (stc.STC_MOD_CHANGESTYLE, "ChangeStyle"),
                 (stc.STC_MOD_CHANGEFOLD, "ChangeFold"),
                 (stc.STC_PERFORMED_USER, "UserFlag"),
                 (stc.STC_PERFORMED_UNDO, "Undo"),
                 (stc.STC_PERFORMED_REDO, "Redo"),
                 (stc.STC_LASTSTEPINUNDOREDO, "Last-Undo/Redo"),
                 (stc.STC_MOD_CHANGEMARKER, "ChangeMarker"),
                 (stc.STC_MOD_BEFOREINSERT, "B4-Insert"),
                 (stc.STC_MOD_BEFOREDELETE, "B4-Delete")
                 ]

        for flag,text in table:
            if flag & modType:
                st = st + text + " "

        if not st:
            st = 'UNKNOWN'

        return st




#----------------------------------------------------------------------

_USE_PANEL = 1

def runTest(frame, nb, log):
    if not _USE_PANEL:
        ed = p = MySTC(nb, -1, log)

    else:
        p = wx.Panel(nb, -1, style=wx.NO_FULL_REPAINT_ON_RESIZE)
        ed = MySTC(p, -1, log)
        s = wx.BoxSizer(wx.HORIZONTAL)
        s.Add(ed, 1, wx.EXPAND)
        p.SetSizer(s)
        p.SetAutoLayout(True)


    ed.SetText(demoText)

    import codecs
    decode = codecs.lookup("utf-8")[1]

    ed.GotoPos(ed.GetLength())
    ed.AddText("\n\nwx.StyledTextCtrl can also do Unicode:\n")
    uniline = ed.GetCurrentLine()
    unitext, l = decode(b'\xd0\x9f\xd0\xb8\xd1\x82\xd0\xbe\xd0\xbd - ' +
                        b'\xd0\xbb\xd1\x83\xd1\x87\xd1\x88\xd0\xb8\xd0\xb9 ' +
                        b'\xd1\x8f\xd0\xb7\xd1\x8b\xd0\xba \xd0\xbf\xd1\x80\xd0\xbe\xd0\xb3\xd1\x80\xd0\xb0\xd0\xbc\xd0\xbc\xd0\xb8\xd1\x80\xd0\xbe\xd0\xb2\xd0\xb0\xd0\xbd\xd0\xb8\xd1\x8f!\n\n')
    ed.AddText('\tRussian: ')
    ed.AddText(unitext)
    ed.GotoPos(0)

    ed.EmptyUndoBuffer()

    # make some styles
    ed.StyleSetSpec(stc.STC_STYLE_DEFAULT, "size:%d,face:%s" % (pb, face3))
    ed.StyleClearAll()
    ed.StyleSetSpec(1, "size:%d,bold,face:%s,fore:#0000FF" % (pb, face1))
    ed.StyleSetSpec(2, "face:%s,italic,fore:#FF0000,size:%d" % (face2, pb))
    ed.StyleSetSpec(3, "face:%s,bold,size:%d" % (face2, pb))
    ed.StyleSetSpec(4, "face:%s,size:%d" % (face1, pb-1))

    if wx.VERSION[0:3] < (4, 1, 0, 0, '')[0:3]:
        def StartStyling(start, mask=255):
            ed.StartStyling(start, mask)
    else:
        def StartStyling(start):
            ed.StartStyling(start)

    # Now set some text to those styles...  Normally this would be
    # done in an event handler that happens when text needs displayed.
    StartStyling(98)
    ed.SetStyling(6, 1)  # set style for 6 characters using style 1

    StartStyling(190)
    ed.SetStyling(20, 2)

    StartStyling(310)
    ed.SetStyling(4, 3)
    ed.SetStyling(2, 0)
    ed.SetStyling(10, 4)


    # line numbers in the margin
    ed.SetMarginType(0, stc.STC_MARGIN_NUMBER)
    ed.SetMarginWidth(0, 22)
    ed.StyleSetSpec(stc.STC_STYLE_LINENUMBER, "size:%d,face:%s" % (pb-2, face1))

    # setup some markers
    ed.SetMarginType(1, stc.STC_MARGIN_SYMBOL)
    ed.MarkerDefine(0, stc.STC_MARK_ROUNDRECT, "#CCFF00", "RED")
    ed.MarkerDefine(1, stc.STC_MARK_CIRCLE, "FOREST GREEN", "SIENNA")
    ed.MarkerDefine(2, stc.STC_MARK_SHORTARROW, "blue", "blue")
    ed.MarkerDefine(3, stc.STC_MARK_ARROW, "#00FF00", "#00FF00")

    # put some markers on some lines
    ed.MarkerAdd(17, 0)
    ed.MarkerAdd(18, 1)
    ed.MarkerAdd(19, 2)
    ed.MarkerAdd(20, 3)
    ed.MarkerAdd(20, 0)

    ed.INDICSTYLE00 = 0
    ed.INDICSTYLE01 = 1
    ed.INDICSTYLE02 = 2

    # and finally, an indicator or two
    # ed.SetIndicatorCurrent(ed.INDICSTYLE00)
    ed.IndicatorSetStyle(ed.INDICSTYLE00, stc.STC_INDIC_SQUIGGLE)
    ed.IndicatorSetForeground(ed.INDICSTYLE00, wx.RED)
    ed.IndicatorSetAlpha(ed.INDICSTYLE00, 64)
    ed.IndicatorSetUnder(ed.INDICSTYLE00, True)
    ed.IndicatorSetStyle(ed.INDICSTYLE01, stc.STC_INDIC_DIAGONAL)
    ed.IndicatorSetForeground(ed.INDICSTYLE01, wx.BLUE)
    ed.IndicatorSetStyle(ed.INDICSTYLE02, stc.STC_INDIC_STRIKE)
    ed.IndicatorSetForeground(ed.INDICSTYLE02, wx.RED)

    ed.SetIndicatorCurrent(ed.INDICSTYLE00)
    ed.IndicatorFillRange(836, 10)
    ed.SetIndicatorCurrent(ed.INDICSTYLE01)
    ed.IndicatorFillRange(846, 8)
    ed.SetIndicatorCurrent(ed.INDICSTYLE02)
    ed.IndicatorFillRange(854, 10)

    ed.AppendText('wxPython %s\n%s\n\n' % (wx.VERSION_STRING, wx.stc.StyledTextCtrl.GetLibraryVersionInfo().GetVersionString()))


    # some test stuff...
    if debug:
        print("GetTextLength(): ", ed.GetTextLength(), len(ed.GetText()))
        print("GetText(): ", repr(ed.GetText()))
        print()
        print("GetStyledText(98, 104): ", repr(ed.GetStyledText(98, 104)), len(ed.GetStyledText(98, 104)))
        print()
        print("GetCurLine(): ", repr(ed.GetCurLine()))
        ed.GotoPos(5)
        print("GetCurLine(): ", repr(ed.GetCurLine()))
        print()
        print("GetLine(1): ", repr(ed.GetLine(1)))
        print()
        ed.SetSelection(25, 35)
        print("GetSelectedText(): ", repr(ed.GetSelectedText()))
        print("GetTextRange(25, 35): ", repr(ed.GetTextRange(25, 35)))
        print("FindText(0, max, 'indicators'): ",
              ed.FindText(0, ed.GetTextLength(), "indicators"))
        # if wx.USE_UNICODE:
        end = ed.GetLength()
        start = ed.PositionFromLine(uniline)
        print("GetTextRange(%d, %d): " % (start, end),)
        print(repr(ed.GetTextRange(start, end)))


    wx.CallAfter(ed.GotoPos, 0)
    return p



#----------------------------------------------------------------------


overview = """\
<html><body>
Once again, no docs yet.  <b>Sorry.</b>  But <a href="data/stc.h.html">this</a>
and <a href="http://www.scintilla.org/ScintillaDoc.html">this</a> should
be helpful.
</body><html>
"""


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])





import sys
import os
import glob
import six

import wx
from wx.svg import SVGimage

#----------------------------------------------------------------------

class SVGBitmapDisplay(wx.Panel):
    """
    A simple panel containing a static box and a rasterized SVG image
    """
    def __init__(self, parent, bmp_size, *args, **kw):
        wx.Panel.__init__(self, parent, *args, **kw)
        self.bmp_size = wx.Size(*bmp_size)
        self.statbmp = wx.StaticBitmap(self, bitmap=wx.Bitmap(*self.bmp_size))
        label='{}x{}'.format(self.bmp_size.width, self.bmp_size.height)
        sbox = wx.StaticBoxSizer(wx.VERTICAL, self, label)
        sbox.Add(self.statbmp)
        self.SetSizer(sbox)
        if not self.IsDoubleBuffered():
            self.SetDoubleBuffered(True)  # Reduce flicker on size event.


    def UpdateSVG(self, svg_filename):
        if six.PY2 and isinstance(svg_filename, unicode):
            svg_filename = svg_filename.encode(sys.getfilesystemencoding())
        img = SVGimage.CreateFromFile(svg_filename)
        bmp = img.ConvertToScaledBitmap(self.bmp_size, self)
        self.statbmp.SetBitmap(bmp)
        #print(bmp.GetSize())


#----------------------------------------------------------------------
ADD_NEW = '[Double-click to Add New File]'

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        self.listbox = wx.ListBox(self, style=wx.LB_SINGLE, size=(250, -1))
        self.listbox.Append(ADD_NEW)
        self.listbox.Append(glob.glob(os.path.join('data', '*.svg')))

        self.updateables = []
        rightSizer = wx.BoxSizer(wx.VERTICAL)
        topRowSizer = wx.BoxSizer(wx.HORIZONTAL)

        # Add a few smallish bitmaps in a row
        for d in [32, 64, 128]:
            sbd = SVGBitmapDisplay(self, (d,d))
            self.updateables.append(sbd)
            topRowSizer.Add(sbd, wx.SizerFlags().Border().Top())

        rightSizer.Add(topRowSizer)

        # and add another, larger one below that row
        sbd = SVGBitmapDisplay(self, (256,256))
        self.updateables.append(sbd)
        rightSizer.Add(sbd)

        self.Sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.Sizer.Add(self.listbox, wx.SizerFlags(1).Border(wx.ALL, 10).Expand())
        self.Sizer.Add(rightSizer, wx.SizerFlags(2).Border(wx.RIGHT|wx.BOTTOM|wx.TOP, 10).Expand())

        self.Bind(wx.EVT_LISTBOX, self.OnSelectItem)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnDClickItem)

        self.listbox.SetSelection(1)

        # Load the first SVG in the list into the static bitmaps
        self.UpdateAll(self.listbox.GetString(1))


    def OnSelectItem(self, evt):
        filename = self.listbox.GetStringSelection()
        if filename != ADD_NEW:
            self.UpdateAll(filename)


    def OnDClickItem(self, evt):
        if self.listbox.GetSelection() == 0:
            with wx.FileDialog(self, "Select SVG file", "data",
                               wildcard="SVG files (*.svg)|*.svg",
                               style=wx.FD_OPEN) as dlg:
                if dlg.ShowModal() == wx.ID_OK:
                    self.listbox.Insert(dlg.GetPath(), 1)
                    self.listbox.SetSelection(1)
                    self.UpdateAll(self.listbox.GetString(1))


    def UpdateAll(self, svg_filename):
        for item in self.updateables:
            item.UpdateSVG(svg_filename)
        self.Layout()

#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------



overview = """<html><body>
<h2><center>SVGImage</center></h2>

The wx.svg.SVGimage class provides the ability to load, parse and render
Scalable Vector Graphics (SVG) files. The advantage of SVG files is that
they can be used to create bitmaps of any size without loss of quality.
<p>
This sample demonstrates rasterizing an SVG image to wx.Bitmaps of various
sizes.

</body></html>
"""



if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




import unittest
from unittests import wtc

import wx.propgrid as pg

#---------------------------------------------------------------------------

class propgrid_Tests(wtc.WidgetTestCase):

    def test_propgrid01(self):
        obj = pg.PGValidationInfo()


    def test_propgrid02(self):
        pg.PG_AUTO_SORT
        pg.PG_HIDE_CATEGORIES
        pg.PG_ALPHABETIC_MODE
        pg.PG_BOLD_MODIFIED
        pg.PG_SPLITTER_AUTO_CENTER
        pg.PG_TOOLTIPS
        pg.PG_HIDE_MARGIN
        pg.PG_STATIC_SPLITTER
        pg.PG_STATIC_LAYOUT
        pg.PG_LIMITED_EDITING
        pg.PG_TOOLBAR
        pg.PG_DESCRIPTION
        pg.PG_NO_INTERNAL_BORDER

        pg.PG_EX_INIT_NOCAT
        pg.PG_EX_NO_FLAT_TOOLBAR
        pg.PG_EX_MODE_BUTTONS
        pg.PG_EX_HELP_AS_TOOLTIPS
        pg.PG_EX_NATIVE_DOUBLE_BUFFERING
        pg.PG_EX_AUTO_UNSPECIFIED_VALUES
        pg.PG_EX_WRITEONLY_BUILTIN_ATTRIBUTES
        pg.PG_EX_HIDE_PAGE_BUTTONS
        pg.PG_EX_MULTIPLE_SELECTION
        pg.PG_EX_ENABLE_TLP_TRACKING
        pg.PG_EX_NO_TOOLBAR_DIVIDER
        pg.PG_EX_TOOLBAR_SEPARATOR

        pg.PG_VFB_STAY_IN_PROPERTY
        pg.PG_VFB_BEEP
        pg.PG_VFB_MARK_CELL
        pg.PG_VFB_SHOW_MESSAGE
        pg.PG_VFB_SHOW_MESSAGEBOX
        pg.PG_VFB_SHOW_MESSAGE_ON_STATUSBAR
        pg.PG_VFB_DEFAULT

        pg.PG_ACTION_INVALID
        pg.PG_ACTION_NEXT_PROPERTY
        pg.PG_ACTION_PREV_PROPERTY
        pg.PG_ACTION_EXPAND_PROPERTY
        pg.PG_ACTION_COLLAPSE_PROPERTY
        pg.PG_ACTION_CANCEL_EDIT
        pg.PG_ACTION_EDIT
        pg.PG_ACTION_PRESS_BUTTON
        pg.PG_ACTION_MAX

        pg.PG_DEFAULT_STYLE
        pg.PGMAN_DEFAULT_STYLE


    def test_propgrid03(self):
        pgrid = pg.PropertyGrid(self.frame)





#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import os


#---------------------------------------------------------------------------

class tooltip_Tests(wtc.WidgetTestCase):

    def test_tooltip(self):
        tip = wx.ToolTip('help message')
        self.frame.SetToolTip(tip)
        tip.Tip
        tip.Window
        tip.Tip = 'new help message'

    def test_tooltipStatics(self):
        wx.ToolTip.Enable(True)
        wx.ToolTip.SetAutoPop(2000)
        wx.ToolTip.SetDelay(2000)
        wx.ToolTip.SetMaxWidth(500)
        wx.ToolTip.SetReshow(2000)

#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx

#---------------------------------------------------------------------------

class platinfo_Tests(wtc.WidgetTestCase):

    def test_platinfo(self):
        pi = wx.PlatformInformation.Get()

        pi.GetBitness()
        pi.GetOperatingSystemId()
        pi.GetPortId()



    def test_platinfoProperties(self):
        pi = wx.PlatformInformation.Get()
        pi.Bitness
        pi.BitnessName
        pi.DesktopEnvironment
        pi.Endianness
        pi.EndiannessName
        pi.LinuxDistributionInfo
        pi.OSMajorVersion
        pi.OSMinorVersion
        pi.OperatingSystemDescription
        pi.OperatingSystemFamilyName
        pi.OperatingSystemId
        pi.OperatingSystemIdName
        pi.PortId
        pi.PortIdName
        pi.PortIdShortName
        pi.ToolkitMajorVersion
        pi.ToolkitMinorVersion


    def test_platinfoFlags(self):
        wx.OS_UNKNOWN
        wx.OS_MAC_OS
        wx.OS_MAC_OSX_DARWIN
        wx.OS_MAC
        wx.OS_WINDOWS_NT
        wx.OS_WINDOWS
        wx.OS_UNIX_LINUX
        wx.OS_UNIX_FREEBSD
        wx.OS_UNIX_OPENBSD
        wx.OS_UNIX_NETBSD
        wx.OS_UNIX_SOLARIS
        wx.OS_UNIX_AIX
        wx.OS_UNIX_HPUX
        wx.OS_UNIX

        wx.PORT_UNKNOWN
        wx.PORT_BASE
        wx.PORT_MSW
        wx.PORT_MOTIF
        wx.PORT_GTK
        wx.PORT_X11
        wx.PORT_MAC
        wx.PORT_COCOA
        wx.PORT_DFB

        wx.ARCH_INVALID
        wx.ARCH_32
        wx.ARCH_64
        wx.ARCH_MAX

        wx.ENDIAN_INVALID
        wx.ENDIAN_BIG
        wx.ENDIAN_LITTLE
        wx.ENDIAN_PDP
        wx.ENDIAN_MAX

#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import wx.richtext

#---------------------------------------------------------------------------

class richtextxml_Tests(wtc.WidgetTestCase):

    def test_richtextxml1(self):
        handler = wx.richtext.RichTextXMLHandler()
        wx.richtext.RichTextBuffer.AddHandler(handler)


#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import time
import wx
import images

#-------------------------------------------------------------------

class MyFrame(wx.Frame):

    def __init__(self, parent, id, log):
        wx.Frame.__init__(self, parent, id, 'Playing with menus', size=(500, 250))
        self.log = log
        self.CenterOnScreen()

        self.CreateStatusBar()
        self.SetStatusText("This is the statusbar")

        tc = wx.TextCtrl(self, -1, """
A bunch of bogus menus have been created for this frame.  You
can play around with them to see how they behave and then
check the source for this sample to see how to implement them.
""", style=wx.TE_READONLY|wx.TE_MULTILINE)

        # Prepare the menu bar
        menuBar = wx.MenuBar()

        # 1st menu from left
        menu1 = wx.Menu()
        menu1.Append(101, "&Mercury", "This the text in the Statusbar")
        menu1.Append(102, "&Venus", "")
        if False:
            # This is how you would create the menu item if you want to
            # change some of the visible attributes.
            item = wx.MenuItem(id=103, text="&Earth", helpString="You may select Earth too")
            item.SetFont(wx.Font(wx.FontInfo(10).Bold()))
            menu1.Append(item)
        else:
            # But we'll just do it the normal way for this sample
            menu1.Append(103, "&Earth", "You may select Earth too")
        menu1.AppendSeparator()
        menu1.Append(104, "&Close", "Close this frame")
        # Add menu to the menu bar
        menuBar.Append(menu1, "&Planets")

        # 2nd menu from left
        menu2 = wx.Menu()
        menu2.Append(201, "Hydrogen")
        menu2.Append(202, "Helium")
        # a submenu in the 2nd menu
        submenu = wx.Menu()
        submenu.Append(2031,"Lanthanium")
        submenu.Append(2032,"Cerium")
        submenu.Append(2033,"Praseodymium")
        menu2.Append(203, "Lanthanides", submenu)
        # Append 2nd menu
        menuBar.Append(menu2, "&Elements")

        menu3 = wx.Menu()
        # Radio items
        menu3.Append(301, "IDLE", "a Python shell using tcl/tk as GUI", wx.ITEM_RADIO)
        menu3.Append(302, "PyCrust", "a Python shell using wxPython as GUI", wx.ITEM_RADIO)
        menu3.Append(303, "psi", "a simple Python shell using wxPython as GUI", wx.ITEM_RADIO)
        menu3.AppendSeparator()
        menu3.Append(304, "project1", "", wx.ITEM_NORMAL)
        menu3.Append(305, "project2", "", wx.ITEM_NORMAL)
        menuBar.Append(menu3, "&Shells")

        menu4 = wx.Menu()
        # Check menu items
        menu4.Append(401, "letters", "abcde...", wx.ITEM_CHECK)
        menu4.Append(402, "digits", "123...", wx.ITEM_CHECK)
        menu4.Append(403, "letters and digits", "abcd... + 123...", wx.ITEM_CHECK)
        menuBar.Append(menu4, "Chec&k")

        menu5 = wx.Menu()
        # Show how to put an icon in the menu item
        item = wx.MenuItem(menu5, 500, "&Smile!\tCtrl+S", "This one has an icon")
        item.SetBitmap(images.Smiles.GetBitmap())
        menu5.Append(item)

        menuitemwithbmp = wx.MenuItem(menu5, wx.ID_ANY, "Submenu with Bitmap")
        # Show how to change the background colour of the menu item
        menuitemwithbmp.SetBackgroundColour(wx.YELLOW)
        # Show how to change the menu item's text colour
        menuitemwithbmp.SetTextColour(wx.BLUE)
        # Show how to change the menu item's font
        menuitemwithbmp.SetFont(wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False, ''))
        submenu = wx.Menu(style=wx.MENU_TEAROFF)
        submenu.Append(wx.MenuItem(menu5, wx.ID_ANY, "Woot!"))
        menuitemwithbmp.SetBitmap(images.book.GetBitmap())
        menuitemwithbmp.SetSubMenu(submenu)
        menu5.Append(menuitemwithbmp)

        # Shortcuts
        menu5.Append(501, "Interesting thing\tCtrl+A", "Note the shortcut!")
        menu5.AppendSeparator()
        menu5.Append(502, "Hello\tShift+H")
        menu5.AppendSeparator()
        menu5.Append(503, "remove the submenu")
        menu6 = wx.Menu()
        menu6.Append(601, "Submenu Item")
        menu5.Append(504, "submenu", menu6)
        menu5.Append(505, "remove this menu")
        menu5.Append(506, "this is updated")
        menu5.Append(507, "insert after this...")
        menu5.Append(508, "...and before this")
        menuBar.Append(menu5, "&Fun")

        self.SetMenuBar(menuBar)

        # Menu events
        self.Bind(wx.EVT_MENU_HIGHLIGHT_ALL, self.OnMenuHighlight)

        self.Bind(wx.EVT_MENU, self.Menu101, id=101)
        self.Bind(wx.EVT_MENU, self.Menu102, id=102)
        self.Bind(wx.EVT_MENU, self.Menu103, id=103)
        self.Bind(wx.EVT_MENU, self.CloseWindow, id=104)

        self.Bind(wx.EVT_MENU, self.Menu201, id=201)
        self.Bind(wx.EVT_MENU, self.Menu202, id=202)
        self.Bind(wx.EVT_MENU, self.Menu2031, id=2031)
        self.Bind(wx.EVT_MENU, self.Menu2032, id=2032)
        self.Bind(wx.EVT_MENU, self.Menu2033, id=2033)

        self.Bind(wx.EVT_MENU, self.Menu301To303, id=301)
        self.Bind(wx.EVT_MENU, self.Menu301To303, id=302)
        self.Bind(wx.EVT_MENU, self.Menu301To303, id=303)
        self.Bind(wx.EVT_MENU, self.Menu304, id=304)
        self.Bind(wx.EVT_MENU, self.Menu305, id=305)

        # Range of menu items
        self.Bind(wx.EVT_MENU_RANGE, self.Menu401To403, id=401, id2=403)

        self.Bind(wx.EVT_MENU, self.Menu500, id=500)
        self.Bind(wx.EVT_MENU, self.Menu501, id=501)
        self.Bind(wx.EVT_MENU, self.Menu502, id=502)
        self.Bind(wx.EVT_MENU, self.TestRemove, id=503)
        self.Bind(wx.EVT_MENU, self.TestRemove2, id=505)
        self.Bind(wx.EVT_MENU, self.TestInsert, id=507)
        self.Bind(wx.EVT_MENU, self.TestInsert, id=508)

        wx.GetApp().Bind(wx.EVT_UPDATE_UI, self.TestUpdateUI, id=506)

    # Methods

    def OnMenuHighlight(self, event):
        # Show how to get menu item info from this event handler
        id = event.GetMenuId()
        item = self.GetMenuBar().FindItemById(id)
        if item:
            text = item.GetItemLabelText()
            help = item.GetHelp()

        # but in this case just call Skip so the default is done
        event.Skip()


    def Menu101(self, event):
        self.log.write('Welcome to Mercury\n')

    def Menu102(self, event):
        self.log.write('Welcome to Venus\n')

    def Menu103(self, event):
        self.log.write('Welcome to the Earth\n')

    def CloseWindow(self, event):
        self.Close()

    def Menu201(self, event):
        self.log.write('Chemical element number 1\n')

    def Menu202(self, event):
        self.log.write('Chemical element number 2\n')

    def Menu2031(self, event):
        self.log.write('Element number 57\n')

    def Menu2032(self, event):
        self.log.write('Element number 58\n')

    def Menu2033(self, event):
        self.log.write('Element number 59\n')

    def Menu301To303(self, event):
        id = event.GetId()
        self.log.write('Event id: %d\n' % id)

    def Menu304(self, event):
        self.log.write('Not yet available\n')

    def Menu305(self, event):
        self.log.write('Still vapour\n')

    def Menu401To403(self, event):
        self.log.write('From a EVT_MENU_RANGE event\n')

    def Menu500(self, event):
        self.log.write('Have a happy day!\n')

    def Menu501(self, event):
        self.log.write('Look in the code how the shortcut has been realized\n')

    def Menu502(self, event):
        self.log.write('Hello from Jean-Michel\n')


    def TestRemove(self, evt):
        mb = self.GetMenuBar()
        submenuItem = mb.FindItemById(601)

        if not submenuItem:
            return

        submenu = submenuItem.GetMenu()
        menu = submenu.GetParent()

        # This works
        #menu.Remove(504)

        # this also works
        menu.RemoveItem(mb.FindItemById(504))

        # This doesn't work, as expected since submenuItem is not on menu
        #menu.RemoveItem(submenuItem)


    def TestRemove2(self, evt):
        mb = self.GetMenuBar()
        mb.Remove(4)


    def TestUpdateUI(self, evt):
        text = time.ctime()
        evt.SetText(text)


    def TestInsert(self, evt):
        theID = 508
        # get the menu
        mb = self.GetMenuBar()
        menuItem = mb.FindItemById(theID)
        menu = menuItem.GetMenu()

        # figure out the position to insert at
        pos = 0

        for i in menu.GetMenuItems():
            if i.GetId() == theID:
                break

            pos += 1

        # now insert the new item
        ID = wx.NewIdRef()
        item = wx.MenuItem(menu, ID, "NewItem " + str(ID))
        menu.InsertItem(pos, item)


#---------------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        b = wx.Button(self, -1, "Show the Menu sample", (50,50))
        self.Bind(wx.EVT_BUTTON, self.OnButton, b)


    def OnButton(self, evt):
        win = MyFrame(self, -1, self.log)
        win.Show(True)


#---------------------------------------------------------------------------


def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#-------------------------------------------------------------------


overview = """\
A demo of using wx.MenuBar and wx.Menu in various ways.

A menu is a popup (or pull down) list of items, one of which may be selected
before the menu goes away (clicking elsewhere dismisses the menu). Menus may be
used to construct either menu bars or popup menus.

A menu item has an integer ID associated with it which can be used to identify
the selection, or to change the menu item in some way. A menu item with a special
identifier -1 is a separator item and doesn't have an associated command but just
makes a separator line appear in the menu.

Menu items may be either normal items, check items or radio items. Normal items
don't have any special properties while the check items have a boolean flag associated
to them and they show a checkmark in the menu when the flag is set. wxWindows
automatically toggles the flag value when the item is clicked and its value may
be retrieved using either IsChecked method of wx.Menu or wx.MenuBar itself or by
using wxEvent.IsChecked when you get the menu notification for the item in question.

The radio items are similar to the check items except that all the other items
in the same radio group are unchecked when a radio item is checked. The radio group
is formed by a contiguous range of radio items, i.e. it starts at the first item of
this kind and ends with the first item of a different kind (or the end of the menu).
Notice that because the radio groups are defined in terms of the item positions
inserting or removing the items in the menu containing the radio items risks to not
work correctly. Finally note that the radio items are only supported under Windows
and GTK+ currently.

"""


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])

